//
//  PhFacebook.m
//  PhFacebook
//
//  Created by Philippe on 10-08-25.
//  Copyright 2010 Philippe Casgrain. All rights reserved.
//

#import "PhFacebook.h"
#import "PhWebViewController.h"
#import "PhAuthenticationToken.h"
#import "PhFacebook_URLs.h"
#import "Debug.h"

#define kFBStoreAccessToken @"FBAStoreccessToken"
#define kFBStoreTokenExpiry @"FBStoreTokenExpiry"
#define kFBStoreAccessPermissions @"FBStoreAccessPermissions"

static NSString* kUserAgent = @"FacebookConnect";
static NSString* kStringBoundary = @"3i2ndDfv2rTHiSisAbouNdArYfORhtTPEefj3q2f";

@implementation PhFacebook

#pragma mark Initialization

- (id) initWithApplicationID: (NSString*) appID delegate: (id) delegate
{
    if ((self = [super init]))
    {
        if (appID)
            _appID = [[NSString stringWithString: appID] retain];
        _delegate = delegate; // Don't retain delegate to avoid retain cycles
        _webViewController = nil;
        _authToken = nil;
        _permissions = nil;
        //DebugLog(@"Initialized with AppID '%@'", _appID);
    }

    return self;
}

- (id)initWithApplicationID:(NSString *)appID withSecret:(NSString *)appSecret delegate:(id)delegate
{
    if ((self = [super init]))
    {
        if (appID)
            _appID = [[NSString stringWithString: appID] retain];
        if (appSecret)
            _appSecret = [[NSString stringWithString: appSecret] retain];
        _delegate = delegate; // Don't retain delegate to avoid retain cycles
        _webViewController = nil;
        _authToken = nil;
        _permissions = nil;
        //DebugLog(@"Initialized with AppID '%@'", _appID);
    }
    
    return self;
}

-(NSString *)getAppId
{
    return _appID;
}

-(NSString *)getAppSecret
{
    return _appSecret;
}

- (void) dealloc
{
    [_appID release];
    [_appSecret release];
    [_webViewController release];
    [_authToken release];
    [super dealloc];
}

- (void) notifyDelegateForToken: (PhAuthenticationToken*) token withError: (NSString*) errorReason
{
    NSMutableDictionary *result = [NSMutableDictionary dictionary];
    if (token)
    {
        // Save it to user defaults
        NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
        [defaults setObject: token.authenticationToken forKey: kFBStoreAccessToken];
        if (token.expiry)
            [defaults setObject: token.expiry forKey: kFBStoreTokenExpiry];
        else
            [defaults removeObjectForKey: kFBStoreTokenExpiry];
        [defaults setObject: token.permissions forKey: kFBStoreAccessPermissions];

        [result setObject: [NSNumber numberWithBool: YES] forKey: @"valid"];
    }
    else
    {
        [result setObject: [NSNumber numberWithBool: NO] forKey: @"valid"];
        [result setObject: errorReason forKey: @"error"];
    }

    if ([_delegate respondsToSelector: @selector(tokenResult:)])
        [_delegate tokenResult: result];
}

#pragma mark Access

- (void) clearToken
{
    [_authToken release];
    _authToken = nil;
}

-(void) invalidateCachedToken
{
    if (_webViewController != nil) {
        [_webViewController cleanCookies];
    }
    
    [self clearToken];

    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults removeObjectForKey:kFBStoreAccessToken];
    [defaults removeObjectForKey: kFBStoreTokenExpiry];
    [defaults removeObjectForKey: kFBStoreAccessPermissions];
}

- (void) setAccessToken: (NSString*) accessToken expires: (NSTimeInterval) tokenExpires permissions: (NSString*) perms
{
    [self clearToken];

    if (accessToken)
        _authToken = [[PhAuthenticationToken alloc] initWithToken: accessToken secondsToExpiry: tokenExpires permissions: perms];
}

- (void) getAccessTokenForPermissions: (NSArray*) permissions cached: (BOOL) canCache isLogon:(BOOL) isLogon
{
    BOOL validToken = NO;
    NSString *scope = [permissions componentsJoinedByString: @","];

    if (canCache && _authToken == nil)
    {
        NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
        NSString *accessToken = [defaults stringForKey: kFBStoreAccessToken];
        NSDate *date = [defaults objectForKey: kFBStoreTokenExpiry];
        NSString *perms = [defaults stringForKey: kFBStoreAccessPermissions];
        if (accessToken && perms)
        {
            // Do not notify delegate yet...
            [self setAccessToken: accessToken expires: [date timeIntervalSinceNow] permissions: perms];
        }
    }

    if ([_authToken.permissions isCaseInsensitiveLike: scope])
    {
        // We already have a token for these permissions; check if it has expired or not
        if (_authToken.expiry == nil || [[_authToken.expiry laterDate: [NSDate date]] isEqual: _authToken.expiry])
            validToken = YES;
    }

    if (validToken)
    {
        [self notifyDelegateForToken: _authToken withError: nil];
    }
    else if (isLogon)
    {
        [self clearToken];

        // Use _webViewController to request a new token
        NSString *authURL;
        if (scope)
            authURL = [NSString stringWithFormat: kFBAuthorizeWithScopeURL, _appID, _appSecret, kFBLoginSuccessURL, scope];
        else
            authURL = [NSString stringWithFormat: kFBAuthorizeURL, _appID, kFBLoginSuccessURL];
      
        if ([_delegate respondsToSelector: @selector(needsAuthentication:forPermissions:)]) 
        {
            if ([_delegate needsAuthentication: authURL forPermissions: scope]) 
            {
                // If needsAuthentication returns YES, let the delegate handle the authentication UI
                return;
            }
        }
      
        // Retrieve token from web page
        if (_webViewController != nil) {
            [_webViewController release];
        }
        _webViewController = [[PhWebViewController alloc] init];
        
        [NSBundle loadNibNamed: @"FacebookBrowser" owner: _webViewController];

        // Prepare window but keep it ordered out. The _webViewController will make it visible
        // if it needs to.
        _webViewController.parent = self;
        _webViewController.permissions = scope;
        
        [_webViewController runSheet:[NSApp mainWindow]];
    
        [_webViewController openURL:authURL];
    
    }
}

- (void) setAccessToken: (NSString*) accessToken expires: (NSTimeInterval) tokenExpires permissions: (NSString*) perms error: (NSString*) errorReason
{
	[self setAccessToken: accessToken expires: tokenExpires permissions: perms];
	[self notifyDelegateForToken: _authToken withError: errorReason];
}

- (NSString*) accessToken
{
    return [[_authToken.authenticationToken copy] autorelease];
}

- (void)utfAppendBody:(NSMutableData *)body data:(NSString *)data {
    [body appendData:[data dataUsingEncoding:NSUTF8StringEncoding]];
}

- (NSString *)mimeTypeForFileAtPath:(NSString *)path
{
	if (![[[[NSFileManager alloc] init] autorelease] fileExistsAtPath:path]) {
		return nil;
	}
	// Borrowed from http://stackoverflow.com/questions/2439020/wheres-the-iphone-mime-type-database
	CFStringRef UTI = UTTypeCreatePreferredIdentifierForTag(kUTTagClassFilenameExtension, (CFStringRef)[path pathExtension], NULL);
    CFStringRef MIMEType = UTTypeCopyPreferredTagWithClass (UTI, kUTTagClassMIMEType);
    CFRelease(UTI);
	if (!MIMEType) {
		return @"application/octet-stream";
	}
    return [NSMakeCollectable(MIMEType) autorelease];
}

static NSString *const kEscapeChars = @"`~!@#$^&*()=+[]\\{}|;':\",/<>?";
-(NSString*) EscapedURLStringFromNSStringWithExtraEscapedChars :(NSString*) inStr :(NSString*)inEscChars
{
	CFStringRef escaped = CFURLCreateStringByAddingPercentEscapes(NULL, (CFStringRef)inStr, NULL, (CFStringRef)inEscChars, kCFStringEncodingUTF8);
    
	return (NSString *)[NSMakeCollectable(escaped) autorelease];			       
}

- (void) sendFacebookRequest: (NSDictionary*) allParams
{
    NSAutoreleasePool *pool = [NSAutoreleasePool new];

    if (_authToken)
    {
        NSString *request = [allParams objectForKey: @"request"];
        NSString *str;
        BOOL postRequest = [[allParams objectForKey: @"postRequest"] boolValue];
        
        NSDictionary* userInfo = [allParams objectForKey:@"userInfo"];
                
        if (postRequest)
        {
            if([request isEqualToString:@"me/videos"]){
                str = [NSString stringWithFormat: kFBGraphApiVideoPostURL, request];
            }else{
                str = [NSString stringWithFormat: kFBGraphApiPostURL, request];
            }
        }
        else
        {
            // Check if request already has optional parameters
            NSString *formatStr = kFBGraphApiGetURL;
            NSRange rng = [request rangeOfString:@"?"];
            if (rng.length > 0)
                formatStr = kFBGraphApiGetURLWithParams;
            str = [NSString stringWithFormat: formatStr, request, _authToken.authenticationToken];
        }

        
        NSDictionary *params = [allParams objectForKey: @"params"];
        NSMutableString *strPostParams = nil;
        if (params != nil) 
        {
            if (postRequest)
            {
                strPostParams = [NSMutableString stringWithFormat: @"%@?access_token=%@", str,_authToken.authenticationToken];
                for (NSString *p in [params allKeys]) 
                {
                    if ([[params valueForKey:p] isKindOfClass:[NSData class]])
                        continue;
                    
                    if ([p isEqualToString:@"source"]) {
                        continue;
                    }
                    
                    NSString* paramvalue = [self EscapedURLStringFromNSStringWithExtraEscapedChars : [params objectForKey: p] :kEscapeChars ];
                    [strPostParams appendFormat: @"&%@=%@", p, paramvalue];

                    
                }
                str = strPostParams;
            }
            else
            {
                NSMutableString *strWithParams = [NSMutableString stringWithString: str];
                for (NSString *p in [params allKeys]) 
                {
                    NSString* paramvalue = [self EscapedURLStringFromNSStringWithExtraEscapedChars : [params objectForKey: p] :kEscapeChars ];
                    [strWithParams appendFormat: @"&%@=%@", p, paramvalue];
                }
                str = strWithParams;
            }
        }
        NSMutableURLRequest *req = [NSMutableURLRequest requestWithURL: [NSURL URLWithString: str]];
        
        //NSLog(@"request: %@",str);
        if (postRequest)
        {
            //Post photo data to http server
            if ([request rangeOfString:@"/photos"].location != NSNotFound  ||
                [request rangeOfString:@"/video"].location != NSNotFound) 
            {
                
                [req setHTTPMethod: @"POST"];
                [req setValue:kUserAgent forHTTPHeaderField:@"User-Agent"];
                
                NSString* contentType = [NSString
                                         stringWithFormat:@"multipart/form-data; boundary=%@", kStringBoundary];
                [req setValue: contentType forHTTPHeaderField: @"Content-Type"];
                
                //NSMutableData *requestData = [NSMutableData dataWithBytes: [strPostParams UTF8String] length: [strPostParams length]];
                NSMutableData *requestData = [NSMutableData data];
                
                NSString *endLine = [NSString stringWithFormat:@"\r\n--%@\r\n", kStringBoundary];
                
                [requestData appendData:[ [NSString stringWithFormat:@"--%@\r\n", kStringBoundary] dataUsingEncoding:NSUTF8StringEncoding]] ;
                
                
                for (id key in [params keyEnumerator]) {
                    
                    if ([[params valueForKey:key] isKindOfClass:[NSData class]]) {
                        
                        //[dataDictionary setObject:[_params valueForKey:key] forKey:key];
                        continue;
                        
                    }
                    
                    [self utfAppendBody:requestData
                                   data:[NSString
                                         stringWithFormat:@"Content-Disposition: form-data; name=\"%@\"\r\n\r\n",
                                         key]];
                    
                    [self utfAppendBody:requestData data:[params valueForKey:key]];
                    
                    [self utfAppendBody:requestData data:endLine];
                }
                
                NSString *imagePath = [params objectForKey:@"source"];
                
                [requestData appendData:[[NSString stringWithFormat:@"Content-Disposition: form-data; filename=\"%@\"\r\n", imagePath] dataUsingEncoding:NSUTF8StringEncoding]];
                
                NSString* mimeType = [self mimeTypeForFileAtPath:imagePath];
                NSString* appendString = [NSString stringWithFormat:@"Content-Type: %@\r\n\r\n",mimeType];
                
                [requestData appendData:[[NSString stringWithString:appendString] dataUsingEncoding:NSUTF8StringEncoding]];
                

                NSData *imageData = [NSData dataWithContentsOfFile:imagePath];
                
                [requestData appendData:[NSData dataWithData:imageData]];
                
                [requestData appendData:[ endLine dataUsingEncoding:NSUTF8StringEncoding]];
               
                [req setHTTPBody: requestData];
                

            }
            else
            {
                NSData *requestData = [NSData dataWithBytes: [strPostParams UTF8String] length: [strPostParams length]];
                [req setHTTPMethod: @"POST"];
                [req setHTTPBody: requestData];
                [req setValue: @"application/x-www-form-urlencoded" forHTTPHeaderField: @"content-type"];               
            }

        }
        
        NSURLResponse *response = nil;
        NSError *error = nil;
        NSData *data = [NSURLConnection sendSynchronousRequest: req returningResponse: &response error: &error];
        NSLog(@"**********************************************");
        NSLog(@"%@",error);
        NSLog(@"%@",response);
        if ([_delegate respondsToSelector: @selector(requestResult:)])
        {
            NSString *str = [[NSString alloc] initWithBytesNoCopy: (void*)[data bytes] length: [data length] encoding:NSASCIIStringEncoding freeWhenDone: NO];

            NSMutableDictionary *result = [NSMutableDictionary dictionaryWithObject: request forKey: @"request"];
            
            [result setObject:str forKey:@"result"];
            
            [result setObject:self forKey:@"sender"];
            
            
            if(error)
            {
                [result setObject:error forKey:@"error"];
            }

            if (data) {
                [result setObject:data forKey:@"raw"];
            }
            
            if (params) {
                [result setObject:params forKey:@"params"];
            }
            
            if (userInfo) {
                [result setObject:userInfo forKey:@"userInfo"];
            }
            
//            if (error) {
//                result = [NSDictionary dictionaryWithObjectsAndKeys:
//                          str, @"result",
//                          request, @"request",
//                          error,@"error",
//                          self, @"sender",
//                          nil];
//            }
//            else
//            {
//                result = [NSDictionary dictionaryWithObjectsAndKeys:
//                    str, @"result",
//                    request, @"request",
//                    data, @"raw",   
//                    params,@"params",
//                    self, @"sender",
//                    nil];
//            }
            
            [_delegate performSelectorOnMainThread:@selector(requestResult:) withObject: result waitUntilDone:YES];
            [str release];
        }
    }
    [pool drain];
}

- (void) sendRequest: (NSString*) request params: (NSDictionary*) params usePostRequest: (BOOL) postRequest
{
    NSMutableDictionary *allParams = [NSMutableDictionary dictionaryWithObject: request forKey: @"request"];
    if (params != nil)
        [allParams setObject: params forKey: @"params"];
        
    [allParams setObject: [NSNumber numberWithBool: postRequest] forKey: @"postRequest"];

	[NSThread detachNewThreadSelector: @selector(sendFacebookRequest:) toTarget: self withObject: allParams];    
}

- (void) sendRequest: (NSString*) request params: (NSDictionary*) params userInfo :(NSDictionary*) userInfo usePostRequest: (BOOL) postRequest
{
    NSMutableDictionary *allParams = [NSMutableDictionary dictionaryWithObject: request forKey: @"request"];
    if (params != nil)
        [allParams setObject: params forKey: @"params"];
    
    [allParams setObject: [NSNumber numberWithBool: postRequest] forKey: @"postRequest"];
    
    [allParams setObject:userInfo forKey:@"userInfo"];
    
	[NSThread detachNewThreadSelector: @selector(sendFacebookRequest:) toTarget: self withObject: allParams];    
}

- (void) sendRequest: (NSString*) request
{
    [self sendRequest: request params: nil usePostRequest: NO];
}

- (void) sendFacebookFQLRequest: (NSString*) query
{
    NSAutoreleasePool *pool = [NSAutoreleasePool new];

    if (_authToken)
    {
        NSString *str = [NSString stringWithFormat: kFBGraphApiFqlURL, [query stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding], _authToken.authenticationToken];

        NSMutableURLRequest *req = [NSMutableURLRequest requestWithURL: [NSURL URLWithString: str]];

        NSURLResponse *response = nil;
        NSError *error = nil;
        NSData *data = [NSURLConnection sendSynchronousRequest: req returningResponse: &response error: &error];

        if ([_delegate respondsToSelector: @selector(requestResult:)])
        {
            //NSString *str = [[NSString alloc] initWithBytesNoCopy: (void*)[data bytes] length: [data length] encoding:NSASCIIStringEncoding freeWhenDone: NO];

            NSDictionary *result = nil;
            if (error) {
                result = [NSDictionary dictionaryWithObjectsAndKeys:
                          data, @"result",
                          query, @"request",
                          error,@"error",
                          self, @"sender",
                          nil];
            }
            else
            {
                result = [NSDictionary dictionaryWithObjectsAndKeys:
                                        data, @"result",
                                        query, @"request",
                                        data, @"raw",
                                        self, @"sender",
                                        nil];
            }
            [_delegate performSelectorOnMainThread:@selector(requestResult:) withObject: result waitUntilDone:YES];
            //[str release];
        }
    }
    [pool drain];
}

- (void) sendFQLRequest: (NSString*) query
{
    [NSThread detachNewThreadSelector: @selector(sendFacebookFQLRequest:) toTarget: self withObject: query];
}

#pragma mark Notifications

- (void) webViewWillShowUI
{
    if ([_delegate respondsToSelector: @selector(willShowUINotification:)])
        [_delegate performSelectorOnMainThread: @selector(willShowUINotification:) withObject: self waitUntilDone: YES];
}

- (void) didDismissUI
{
    if ([_delegate respondsToSelector: @selector(didDismissUI:)])
        [_delegate performSelectorOnMainThread: @selector(didDismissUI:) withObject: self waitUntilDone: YES];
}

@end
